function [Fitted,Residual,Median_est_fit,Median_est_residual]=varfitandresidualMeanCorr(X,D,K,Pires,Phires,IncludeConstant,ThinOutFreq)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	Estimating the in-sample fit of the Bayesian VAR model
%
% MODEL:    x_{t}-PHI*d_{t}=PI_{1}*(x_{t-1}-PHI*d_{t})+...+PI_{K}*(x_{t-K}-PHI*d_{t})+eps_{t}
%									
% INPUT:	X: 		    (T-by-p) matrix with T observations on p modelled variables.
%			D: 		    (T-by-q) matrix with T observations on q
%			             non-modelled variables. The program adds a column of ones for the constant in the process
%			K:          (scalar) number of lags in the VAR.
%           Pires:      (niter-by-(p^2)*k) matrix of draws from the posterior of
%                       VAR coefficients.
%           Phires:     (niter-by-p*q) matrix of draws from the posterior of Phi
%
% OUTPUT:   Fitted      (T-p-niter) array with fitted values of the
%                       timeseries. Fitted{i} contains the fit based on the ith
%                       posterior draw.
%           Residual    (T-p-niter) array with residuals values of the
%                       timeseries. Fitted{i} contains the residuals based on the ith
%                       posterior draw.
%           median_est_fit  (T-by-p) matrix of estimated fitted values
%                       based on the posterior median of the model parameters.
%           Median_est_residual (T-by-p) matrix of estimated residuals
%                       based on the posterior median of the model parameters.
%
% AUTHOR:	Mattias Villani, Research Department, Sveriges Riksbank and Department of Statistics, Stockholm University.
%		    E-mail: mattias.villani@riksbank.se.
%
% DATE:		2003-10-30
% REVISED:  2004-06-24
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transform data to multivariate regression form: Y=XB+E.
[T,p]=size(X);
if IncludeConstant>0
    D=[D ones(T,1)];
end
[Yreduced,X1,DReduced]=transform_to_multireg(X,D,K);
[Yphi,Xphi]=datmanip_cond_on_phi(X,D,invvec(Phires(1,:)',p),K);
[T,p]=size(Yphi);

% Thining out the posterior sample
niter=size(Pires,1);
IterLoop=ceil(1:ThinOutFreq:niter);
niter=length(IterLoop);
Fitted=zeros(T,p,niter);
Residual=zeros(T,p,niter);

% Computes posterior distribution of fitted values
Count=0;
for i=IterLoop
    Count=Count+1;
    Phi=invvec(Phires(Count,:)',p);
    [Yphi,Xphi]=datmanip_cond_on_phi(X,D,Phi,K);
    Pi=invvec(Pires(Count,:)',p*K);
    Fitted(:,:,Count)=DReduced*Phi'+Xphi*Pi;
    Residual(:,:,Count)=Yreduced-Fitted(:,:,Count);
end

% Computes estimated fitted values using the posterior median
Median_est_fit=zeros(T,p);
Median_est_residual=zeros(T,p);
medianPi=invvec(median(Pires)',p*K);
medianPhi=invvec(median(Phires)',p);
[Yphi,Xphi]=datmanip_cond_on_phi(X,D,medianPhi,K);
for h=1:T
    Median_est_fit(h,:)=DReduced(h,:)*medianPhi'+Xphi(h,:)*medianPi;
end
Median_est_residual=Yreduced-Median_est_fit;